/*

TODO
- Display a custom webpage when there is an error; something that is actually helpful to the user (we already bypass the privoxy error page)
- Foxtor needs proper localization
- Backward compatibility with Firefox 1.0.x (maybe)
- Appears to be a bug when locking "network.proxy.type". hmmm how to overcome.
- Ability for the user to customize the foxtor icon. This is a good design practice and Cynthia made a bunch of super icons that shouldn't go to waste. 
- Other?

Troubleshooting:
- if privoxy isn't running, message is, "Can't connect, is Pricoxy running?"
- if no network connectivity isn't running, message is, "Can't connect, is Pricoxy running?"

Change Log
v0.7.3
- Removed debug lines from default/preferences file; parameters that seemed to cause an error for some

v0.7.2
- fixed support for Firefox 2.0.0.*

v0.7.1
- support for Firefox 2.0.0.*
- Added Collin Jackson's history blocker code: It's a wrapper around the browser's history that bypasses the real history when the Foxtor status pref is at least 300 ("masked"). When active, it prevents both reading and writing into the history, so visited links render as unvisited, and new pages don't show up in the history. Because it protects history within a single browser session, and disables visited link coloring, this protection is more powerful than the preferences-based approach currently used FoxTor. In particular, it ensures that websites you visit while masked cannot use CSS and/or JavaScript to figure out what you were doing while unmasked, and vice versa.

v0.7
- Geographic information now comes from vidalia-project.net. Thanks Matt! This is a big deal, because it makes changing to masked state much quicker, and (I think) more authoritative.

- Foxtor, now with 2 Cookie Jars!: When becoming unmasked, cookies are set per normal. When masked, unmasked cookie jar is saved and new cookie jar is created. When becoming unmasked, masked cookie jar is destroyed and unmasked cookie jar is restored! Thanks to Collin Jackson for this code.

- Added the ability to reset masked and unmasked settings to their original value (just in case people muck up the settings and don't know what to fix).

- "More Information" page now automatically displays masked and unmasked geoIP information. No more "Get Details" button.

- Simplified status bar and popup menu, that tries to be informative when an error occurs. e.g. "Can't connet. Is Privoxy running?"

v0.6
- First release


Acknowledgements:
- Thanks to Roger Dingledine for Tor
- Thanks to Matt Edmann and the vidalia folks for their suggestions
- Thanks to Collin Jackson for his Cookie and History work
- Thanks to Mel Reyes for his great book on Hacking Firefox
- Thanks to Cynthia Kuo for her great UI and image work
- Thanks to the CMU CUPS folks for the initial FoxTor design
- Thanks to Kok-Chie Daniel Pu for his great ideas

*/
 
var geoIPURL = "http://geoip.vidalia-project.net/cgi-bin/geoip";
var foxTorPrefs = Components.classes["@mozilla.org/preferences-service;1"].
                  getService(Components.interfaces.nsIPrefBranch);

/*
// Log errors in chrome file to the Javascript Console
user_pref( "javascript.options.showInConsole", true);
// Disable XUL cache, so chrome changes do not require a restart
user_pref( "nglayout.debug.disable_xul_cache", true);
// dump() function outputs to console if application started with -console flag
user_pref( "browser.dom.window.dump.enabled", true);
// Enable strict javascript warnings, to produce clean extensions
user_pref( "javascript.options.strict", true);
*/

function aboutFoxtor() {
    window.openDialog("chrome://foxtor/content/foxtorAbout.xul", "_blank", "chrome, centerscreen=yes");
    return true;
}

function initFoxtor() {
    // careful, this function runs each second 
    var status = getStatus();
    setStatus(status);
    // this ensures that every window displays the current state
    setTimeout(initFoxtor, 1000);
}

function getStatus() {
    return foxTorPrefs.getIntPref("extensions.foxtor.status");
}

/*  Valid States are:
    100: unmasked: user is using their normal (identifiable) proxy settings
    200: masking: user is attempting to put on their mask
    300: masked: user is masked, operating through privoxy, tor
    400: error condition. status stored in extensions.foxtor.errorStatus
*/

function setStatus(status) {
    if (status == 100) { //unmasked
        foxTorPrefs.setIntPref("extensions.foxtor.status", 100);
        foxTorPrefs.setCharPref("extensions.foxtor.torip", "");
        foxTorPrefs.setCharPref("extensions.foxtor.tornode", "");

        var ledElement = document.getElementById('foxtor-statusbar-panel');
        ledElement.setAttribute('torstate', 'unmasked');
        ledElement.setAttribute('label', 'You are now Unmasked');

        var ledElement = document.getElementById('foxtor-statusbar-panelimage');
        ledElement.setAttribute('torstate', 'unmasked');
        ledElement.setAttribute('tooltiptext', 'Click here to Mask your identity');
    }
    else if (status == 200) {  //masking
        foxTorPrefs.setIntPref("extensions.foxtor.status", 200);
        var ledElement = document.getElementById('foxtor-statusbar-panel');
        ledElement.setAttribute('torstate', 'masking');
        ledElement.setAttribute('label', 'Putting on your Mask...');
    
        var ledElement = document.getElementById('foxtor-statusbar-panelimage');
        ledElement.setAttribute('torstate', 'masking');
        ledElement.setAttribute('tooltiptext', 'Putting on your Mask...');
    } 
    else if (status == 300) {   // masked
        foxTorPrefs.setIntPref("extensions.foxtor.status", 300);
        var ledElement = document.getElementById('foxtor-statusbar-panel');
        ledElement.setAttribute('torstate', 'masked');
        ledElement.setAttribute('label', 'You are now Masked');
    
        var ledElement = document.getElementById('foxtor-statusbar-panelimage');
        ledElement.setAttribute('torstate', 'masked');
        ledElement.setAttribute('tooltiptext', 'Click here to Remove your Mask');
    } 
    else {  // error state
        foxTorPrefs.setIntPref("extensions.foxtor.status", 400);
        var msg = foxTorPrefs.getCharPref("extensions.foxtor.errorStatus");

        var ledElement = document.getElementById('foxtor-statusbar-panel');
        ledElement.setAttribute('torstate', 'error');
        ledElement.setAttribute('label', msg);

        var ledElement = document.getElementById('foxtor-statusbar-panelimage');
        ledElement.setAttribute('torstate', 'error');
        ledElement.setAttribute('tooltiptext', msg);
    }
}

// options for checking http path through Tor:
// the Harvard test page:
// var pageURL = 'http://serifos.eecs.harvard.edu/cgi-bin/ipaddr.pl?tor=1';
// or, even better, the "hidden" Tor services
// var pageURL = 'http://torlandypjxiligx.onion/'; 

// Here we're going to compare the IPs from a Masked and Unmasked request.
// We do this by requesting geoIP info from vidalia-project.net. 
// We're comparing result of the request through no proxy (unmasked), then
// through privoxy/tor (masked). If the IPs (our IP as seen by the server) are different, 
// then we know the Tor connection is working. (yes, there is a fringe case, but we'll ignore for now)

function getGeoInfo(state) {
    dump('calling httprequest: ' + state + '\n');
    var req = new XMLHttpRequest();
    req.state = state;
    req.open('GET', geoIPURL, true); 
    req.onreadystatechange  = function() {
        if (req.readyState == 4) { // 4 means load has completed
          try {
            if (req.status == 200) {
                var text = parseGeoResult(req.responseText);
                if (text == "error") {
                    var msg = "Didn't get proper response from GeoIP";
                    dump(msg + "\n");
                    foxTorPrefs.setCharPref("extensions.foxtor.errorStatus", msg);
                    setStatus(400);
                }
                else {  // got response
                    dump("Got geo info (" + req.state + "): " + text);
                    if (req.state == "unmasked") {
                        foxTorPrefs.setCharPref("extensions.foxtor.unmaskedGeoInfo", text);
                        saveUnmaskedPrefs();    // save preferences
                        restoreMaskedPrefs();   // restore (set) masked preferences
                        // here's where we call this function again. This works because we maintain state
                        // The first call hits this condition, the second call hits the "else" condition
                        getGeoInfo("masked");
                        // I think this is pretty cool, because we're using only asynchronous
                        // (i.e. NON-blocking) calls and that means less annoyance to the user
                        saveMaskedPrefs();
                        restoreUnmaskedPrefs();  // restore original preferences
                    }
                    else {  // we must be in the 'masked' state 
                        foxTorPrefs.setCharPref("extensions.foxtor.maskedGeoInfo", text);
                        if (getUnmaskedIP().toString() == getMaskedIP().toString()) {
                            // same IPs means we're not masked
                            var msg = "Masked and Unmasked IP addresses are the same";
                            dump(msg + "\n");
                            foxTorPrefs.setCharPref("extensions.foxtor.errorStatus", msg);
                            setStatus(400);
                        }
                        else {
                            // different IPs are good, we must be masked
                            // This is where we actually change state to MASKED
                            dump("IPs are different! connection is working\n");
                            saveUnmaskedPrefs();    // first, save their preferences
                            restoreMaskedPrefs();   // restore (set) masked preferences
                            setStatus(300);         // now go Masked
                        }
                    } // end: if (req.state == "unmasked")
                } 
            } // end: if (req.status == 200)
            else if (req.status == 503) {
                // this is a response from privoxy and generally means tor isn't working
                var msg = "Can\'t connect. Is Tor running?";
                dump(msg + "\n");
                foxTorPrefs.setCharPref("extensions.foxtor.errorStatus", msg);
                setStatus(400);
            }    
            else {  // some other return status
                // communication error either due to tor, or privoxy not running
                // or no network connectivity
                var msg = "Is there netork connectivity? (" + req.status + ")";
                dump(msg + "\n");
                foxTorPrefs.setCharPref("extensions.foxtor.errorStatus", msg);
                setStatus(400);
            }
          } // end try
          catch (e) {
            var msg = "Can\'t connect. Is Privoxy running?";
            dump(msg + "\n");
            foxTorPrefs.setCharPref("extensions.foxtor.errorStatus", msg);
            setStatus(400);
          }
        } // close if req.readyState
        else {
            // different readyState
        }
    } // close onreadystatechange
    req.send(null);
}

function parseGeoResult(text) {
    // we're looking for a response like this:
    // 67.171.66.171,Pittsburgh,PA,US,40.4411,-79.9964
    // in case city, state info isn't available, it will look like this
    // 67.171.66.171,,,US,40.4411,-79.9964
    var textArray = text.split(",");
    var ipPattern = /\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}/;
    var ip = ipPattern.exec(textArray[0]);
    if (ip != null) { // ip will have null on non match
        //text = text.replace(",,," , ",unknown,"); // replace blank data with something more meaningful ;)
        return text;
    }
    else { // geoip page loaded but didn't return what we wanted
        return "error";
    }
}  


function getUnmaskedIP() {
    var info = foxTorPrefs.getCharPref("extensions.foxtor.unmaskedGeoInfo");
    return info.split(",")[0];
}
function getUnmaskedCity() {
    var info = foxTorPrefs.getCharPref("extensions.foxtor.unmaskedGeoInfo");
    return info.split(",")[1] + ", " + info.split(",")[2];
}
function getUnmaskedCountry() {
    var info = foxTorPrefs.getCharPref("extensions.foxtor.unmaskedGeoInfo");
    return info.split(",")[3];
}

function getMaskedIP() {
    var info = foxTorPrefs.getCharPref("extensions.foxtor.maskedGeoInfo");
    return info.split(",")[0];
}
function getMaskedCity() {
    var info = foxTorPrefs.getCharPref("extensions.foxtor.maskedGeoInfo");
    var city = info.split(",")[1];
    var state = info.split(",")[2];

    //dump("city [" + city + "], state [" + state + "]");

    if (city == "" && state != "") {
        return state;
    }
    else if (city != "" && state != "") {   // hopefully the typical case
        return info.split(",")[1] + ", " + info.split(",")[2];
    }        
    else { // this is where both city and state are unknown
        return "unknown";
    }

}
function getMaskedCountry() {
    var info = foxTorPrefs.getCharPref("extensions.foxtor.maskedGeoInfo");
    return info.split(",")[3];
}

// Here's what we're doing:

// 1: when going from unmasked to masked:
// 1a: save current preferences to extensions.foxtor.*
// 1b: restore masked preferences

// 2: when going from masked to unmasked:
// 2a: save current preferences to extensions.foxtor.masked.*
// 2b: restore unmasked preferences


// 1a: placeholder for all preferences
function saveUnmaskedPrefs() {
    // this saves the users current (unmasked) privacy preferences
    // so we chance change them to our 'high' privacy settings
    foxTorPrefs.setIntPref("extensions.foxtor.browser.history_expire_days", foxTorPrefs.getIntPref("browser.history_expire_days"));
    foxTorPrefs.setIntPref("extensions.foxtor.browser.sessionhistory.max_entries", foxTorPrefs.getIntPref("browser.sessionhistory.max_entries"));
    foxTorPrefs.setBoolPref("extensions.foxtor.browser.formfill.enable", foxTorPrefs.getBoolPref("browser.formfill.enable"));
    foxTorPrefs.setBoolPref("extensions.foxtor.signon.rememberSignons", foxTorPrefs.getBoolPref("signon.rememberSignons"));
    foxTorPrefs.setIntPref("extensions.foxtor.browser.download.manager.retention", foxTorPrefs.getIntPref("browser.download.manager.retention"));
    foxTorPrefs.setIntPref("extensions.foxtor.network.cookie.cookieBehavior", foxTorPrefs.getIntPref("network.cookie.cookieBehavior"));
    foxTorPrefs.setIntPref("extensions.foxtor.network.cookie.lifetimePolicy", foxTorPrefs.getIntPref("network.cookie.lifetimePolicy"));
    foxTorPrefs.setIntPref("extensions.foxtor.browser.cache.disk.capacity", foxTorPrefs.getIntPref("browser.cache.disk.capacity"));
    foxTorPrefs.setBoolPref("extensions.foxtor.browser.cache.disk.enable", foxTorPrefs.getBoolPref("browser.cache.disk.enable"));
    foxTorPrefs.setBoolPref("extensions.foxtor.xpinstall.enabled", foxTorPrefs.getBoolPref("xpinstall.enabled"));

        foxTorPrefs.setIntPref("extensions.foxtor.permissions.default.image", foxTorPrefs.getIntPref("permissions.default.image"));
        foxTorPrefs.setBoolPref("extensions.foxtor.app.update.enabled", foxTorPrefs.getBoolPref("app.update.enabled"));
        foxTorPrefs.setBoolPref("extensions.foxtor.browser.search.update", foxTorPrefs.getBoolPref("browser.search.update"));
        foxTorPrefs.setBoolPref("extensions.foxtor.extensions.update.enabled", foxTorPrefs.getBoolPref("extensions.update.enabled"));

    foxTorPrefs.setBoolPref("extensions.foxtor.security.enable_java", foxTorPrefs.getBoolPref("security.enable_java"));
    foxTorPrefs.setBoolPref("extensions.foxtor.javascript.enabled", foxTorPrefs.getBoolPref("javascript.enabled"));

    foxTorPrefs.setIntPref("extensions.foxtor.network.proxy.type", foxTorPrefs.getIntPref("network.proxy.type"));
    foxTorPrefs.setCharPref("extensions.foxtor.network.proxy.http", foxTorPrefs.getCharPref("network.proxy.http"));
    foxTorPrefs.setIntPref("extensions.foxtor.network.proxy.http_port", foxTorPrefs.getIntPref("network.proxy.http_port"));
    foxTorPrefs.setCharPref("extensions.foxtor.network.proxy.no_proxies_on", foxTorPrefs.getCharPref("network.proxy.no_proxies_on"));
    foxTorPrefs.setBoolPref("extensions.foxtor.network.proxy.share_proxy_settings", foxTorPrefs.getBoolPref("network.proxy.share_proxy_settings"));

    // part of saving the unmasked preferences means saving the unmasked cookie jar
    var selector = 
      Components.classes["@stanford.edu/cookie-jar-selector;1"]  
                .getService(Components.interfaces.nsISupports)
                .wrappedJSObject;
    selector.saveCookies("unmasked");  
    selector.clearCookies();

    return true;
}

// 1b: restore (set) masked preferences
function restoreMaskedPrefs() {
    foxTorPrefs.setIntPref("browser.history_expire_days", foxTorPrefs.getIntPref("extensions.foxtor.masked.browser.history_expire_days"));
    foxTorPrefs.setIntPref("browser.sessionhistory.max_entries", foxTorPrefs.getIntPref("extensions.foxtor.masked.browser.sessionhistory.max_entries"));
    foxTorPrefs.setBoolPref("browser.formfill.enable", foxTorPrefs.getBoolPref("extensions.foxtor.masked.browser.formfill.enable"));
    foxTorPrefs.setBoolPref("signon.rememberSignons", foxTorPrefs.getBoolPref("extensions.foxtor.masked.signon.rememberSignons"));
    foxTorPrefs.setIntPref("browser.download.manager.retention", foxTorPrefs.getIntPref("extensions.foxtor.masked.browser.download.manager.retention"));
    foxTorPrefs.setIntPref("network.cookie.cookieBehavior", foxTorPrefs.getIntPref("extensions.foxtor.masked.network.cookie.cookieBehavior"));
    foxTorPrefs.setIntPref("network.cookie.lifetimePolicy", foxTorPrefs.getIntPref("extensions.foxtor.masked.network.cookie.lifetimePolicy"));
    foxTorPrefs.setIntPref("browser.cache.disk.capacity", foxTorPrefs.getIntPref("extensions.foxtor.masked.browser.cache.disk.capacity"));
    foxTorPrefs.setBoolPref("browser.cache.disk.enable", foxTorPrefs.getBoolPref("extensions.foxtor.masked.browser.cache.disk.enable"));
    foxTorPrefs.setBoolPref("xpinstall.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.masked.xpinstall.enabled"));
    foxTorPrefs.setBoolPref("security.enable_java", foxTorPrefs.getBoolPref("extensions.foxtor.masked.security.enable_java"));
    foxTorPrefs.setBoolPref("javascript.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.masked.javascript.enabled"));

        foxTorPrefs.setIntPref("permissions.default.image", foxTorPrefs.getIntPref("extensions.foxtor.masked.permissions.default.image"));    
        foxTorPrefs.setBoolPref("app.update.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.masked.app.update.enabled"));     
        foxTorPrefs.setBoolPref("browser.search.update", foxTorPrefs.getBoolPref("extensions.foxtor.masked.browser.search.update"));   
        foxTorPrefs.setBoolPref("extensions.update.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.masked.extensions.update.enabled"));

     // enable proxy
    foxTorPrefs.setIntPref("network.proxy.type", foxTorPrefs.getIntPref("extensions.foxtor.masked.network.proxy.type"));
    // localhost
    foxTorPrefs.setCharPref("network.proxy.http", foxTorPrefs.getCharPref("extensions.foxtor.masked.network.proxy.http"));
    // privoxy port
    foxTorPrefs.setIntPref("network.proxy.http_port", foxTorPrefs.getIntPref("extensions.foxtor.masked.network.proxy.http_port"));
    // no exceptions
    foxTorPrefs.setCharPref("network.proxy.no_proxies_on", foxTorPrefs.getCharPref("extensions.foxtor.masked.network.proxy.no_proxies_on"));
    // all protocols use proxy settings
    foxTorPrefs.setBoolPref("network.proxy.share_proxy_settings", foxTorPrefs.getBoolPref("extensions.foxtor.masked.network.proxy.share_proxy_settings"));

    return true;
}

// 2a: 
function saveMaskedPrefs() {
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.history_expire_days", foxTorPrefs.getIntPref("browser.history_expire_days"));
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.sessionhistory.max_entries", foxTorPrefs.getIntPref("browser.sessionhistory.max_entries"));
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.browser.formfill.enable", foxTorPrefs.getBoolPref("browser.formfill.enable"));
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.signon.rememberSignons", foxTorPrefs.getBoolPref("signon.rememberSignons"));
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.download.manager.retention", foxTorPrefs.getIntPref("browser.download.manager.retention"));
    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.cookie.cookieBehavior", foxTorPrefs.getIntPref("network.cookie.cookieBehavior"));
    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.cookie.lifetimePolicy", foxTorPrefs.getIntPref("network.cookie.lifetimePolicy"));
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.cache.disk.capacity", foxTorPrefs.getIntPref("browser.cache.disk.capacity"));
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.browser.cache.disk.enable", foxTorPrefs.getBoolPref("browser.cache.disk.enable"));
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.xpinstall.enabled", foxTorPrefs.getBoolPref("xpinstall.enabled"));
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.security.enable_java", foxTorPrefs.getBoolPref("security.enable_java"));
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.javascript.enabled", foxTorPrefs.getBoolPref("javascript.enabled"));

        foxTorPrefs.setIntPref("extensions.foxtor.masked.permissions.default.image", foxTorPrefs.getIntPref("permissions.default.image"));    
        foxTorPrefs.setBoolPref("extensions.foxtor.masked.app.update.enabled", foxTorPrefs.getBoolPref("app.update.enabled"));     
        foxTorPrefs.setBoolPref("extensions.foxtor.masked.browser.search.update", foxTorPrefs.getBoolPref("browser.search.update"));   
        foxTorPrefs.setBoolPref("extensions.foxtor.masked.extensions.update.enabled", foxTorPrefs.getBoolPref("extensions.update.enabled"));

    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.proxy.type", foxTorPrefs.getIntPref("network.proxy.type")); // enable proxy
    foxTorPrefs.setCharPref("extensions.foxtor.masked.network.proxy.http", foxTorPrefs.getCharPref("network.proxy.http"));
    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.proxy.http_port", foxTorPrefs.getIntPref("network.proxy.http_port")); // privoxy port
    foxTorPrefs.setCharPref("extensions.foxtor.masked.network.proxy.no_proxies_on", foxTorPrefs.getCharPref("network.proxy.no_proxies_on")); // no exceptions
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.network.proxy.share_proxy_settings", foxTorPrefs.getBoolPref("network.proxy.share_proxy_settings")); // all protocols use proxy settings

    return true;
}

// 2b:
function restoreUnmaskedPrefs() {
    foxTorPrefs.setIntPref("browser.history_expire_days", foxTorPrefs.getIntPref("extensions.foxtor.browser.history_expire_days"));
    foxTorPrefs.setIntPref("browser.sessionhistory.max_entries", foxTorPrefs.getIntPref("extensions.foxtor.browser.sessionhistory.max_entries"));
    foxTorPrefs.setBoolPref("browser.formfill.enable", foxTorPrefs.getBoolPref("extensions.foxtor.browser.formfill.enable"));
    foxTorPrefs.setBoolPref("signon.rememberSignons", foxTorPrefs.getBoolPref("extensions.foxtor.signon.rememberSignons"));
    foxTorPrefs.setIntPref("browser.download.manager.retention", foxTorPrefs.getIntPref("extensions.foxtor.browser.download.manager.retention"));
    foxTorPrefs.setIntPref("network.cookie.cookieBehavior", foxTorPrefs.getIntPref("extensions.foxtor.network.cookie.cookieBehavior"));
    foxTorPrefs.setIntPref("network.cookie.lifetimePolicy", foxTorPrefs.getIntPref("extensions.foxtor.network.cookie.lifetimePolicy"));
    foxTorPrefs.setIntPref("browser.cache.disk.capacity", foxTorPrefs.getIntPref("extensions.foxtor.browser.cache.disk.capacity"));
    foxTorPrefs.setBoolPref("browser.cache.disk.enable", foxTorPrefs.getBoolPref("extensions.foxtor.browser.cache.disk.enable"));
    foxTorPrefs.setBoolPref("xpinstall.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.xpinstall.enabled"));
    foxTorPrefs.setBoolPref("security.enable_java", foxTorPrefs.getBoolPref("extensions.foxtor.security.enable_java"));
    foxTorPrefs.setBoolPref("javascript.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.javascript.enabled"));

        foxTorPrefs.setIntPref("permissions.default.image", foxTorPrefs.getIntPref("extensions.foxtor.permissions.default.image"));
        foxTorPrefs.setBoolPref("app.update.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.app.update.enabled"));
        foxTorPrefs.setBoolPref("browser.search.update", foxTorPrefs.getBoolPref("extensions.foxtor.browser.search.update"));
        foxTorPrefs.setBoolPref("extensions.update.enabled", foxTorPrefs.getBoolPref("extensions.foxtor.extensions.update.enabled"));

    foxTorPrefs.setIntPref("network.proxy.type", foxTorPrefs.getIntPref("extensions.foxtor.network.proxy.type"));
    foxTorPrefs.setCharPref("network.proxy.http", foxTorPrefs.getCharPref("extensions.foxtor.network.proxy.http"));
    foxTorPrefs.setIntPref("network.proxy.http_port", foxTorPrefs.getIntPref("extensions.foxtor.network.proxy.http_port"));
    foxTorPrefs.setCharPref("network.proxy.no_proxies_on", foxTorPrefs.getCharPref("extensions.foxtor.network.proxy.no_proxies_on"));
    foxTorPrefs.setBoolPref("network.proxy.share_proxy_settings", foxTorPrefs.getBoolPref("extensions.foxtor.network.proxy.share_proxy_settings"));

    // now that we've restored the prefs, we need to restore the unmasked cookie jar
    var selector = 
      Components.classes["@stanford.edu/cookie-jar-selector;1"]  
                .getService(Components.interfaces.nsISupports)
                .wrappedJSObject;
    selector.clearCookies();
    selector.loadCookies("unmasked", true); 
        
    return true;
}

//these next two functions are useful when a user screws up their masked or unmasked settings
// and wants to revert to the default state
function resetMaskedSettings() {
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.history_expire_days",0);
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.sessionhistory.max_entries",0);
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.browser.formfill.enable",false);
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.signon.rememberSignons",false);
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.download.manager.retention",0);
    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.cookie.cookieBehavior",2);
    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.cookie.lifetimePolicy",0);
    foxTorPrefs.setIntPref("extensions.foxtor.masked.browser.cache.disk.capacity",0);
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.browser.cache.disk.enable",false);

    foxTorPrefs.setBoolPref("extensions.foxtor.masked.xpinstall.enabled",false);
    foxTorPrefs.setIntPref("extensions.foxtor.masked.permissions.default.image",3);    
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.security.enable_java",false);
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.javascript.enabled",false);

    foxTorPrefs.setBoolPref("extensions.foxtor.masked.app.update.enabled",false);     
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.browser.search.update",false);   
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.extensions.update.enabled",false);

    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.proxy.type",1); // enable proxy
    foxTorPrefs.setCharPref("extensions.foxtor.masked.network.proxy.http","localhost");
    foxTorPrefs.setIntPref("extensions.foxtor.masked.network.proxy.http_port",8118); // privoxy port
    foxTorPrefs.setCharPref("extensions.foxtor.masked.network.proxy.no_proxies_on",""); // no exceptions
    foxTorPrefs.setBoolPref("extensions.foxtor.masked.network.proxy.share_proxy_settings",true); // all protocols use proxy settings

    restoreMaskedPrefs();
    return true;
}

function resetUnmaskedSettings() {
    foxTorPrefs.setIntPref("extensions.foxtor.browser.history_expire_days",9);
    foxTorPrefs.setIntPref("extensions.foxtor.browser.sessionhistory.max_entries",50); 
    foxTorPrefs.setBoolPref("extensions.foxtor.browser.formfill.enable",true);
    foxTorPrefs.setBoolPref("extensions.foxtor.signon.rememberSignons",true);
    foxTorPrefs.setIntPref("extensions.foxtor.browser.download.manager.retention",2);
    foxTorPrefs.setIntPref("extensions.foxtor.network.cookie.cookieBehavior",0);
    foxTorPrefs.setIntPref("extensions.foxtor.network.cookie.lifetimePolicy",0);
    foxTorPrefs.setIntPref("extensions.foxtor.browser.cache.disk.capacity",50000);
    foxTorPrefs.setBoolPref("extensions.foxtor.browser.cache.disk.enable",true);

    foxTorPrefs.setBoolPref("extensions.foxtor.xpinstall.enabled",true);
    foxTorPrefs.setIntPref("extensions.foxtor.permissions.default.image",1);    
    foxTorPrefs.setBoolPref("extensions.foxtor.security.enable_java",true);
    foxTorPrefs.setBoolPref("extensions.foxtor.javascript.enabled",true);

    foxTorPrefs.setBoolPref("extensions.foxtor.app.update.enabled",true);     
    foxTorPrefs.setBoolPref("extensions.foxtor.browser.search.update",true);   
    foxTorPrefs.setBoolPref("extensions.foxtor.extensions.update.enabled",true);

    // we may remove this as not every user may be bypassing proxies 
    foxTorPrefs.setIntPref("extensions.foxtor.network.proxy.type",0); // enable/disable proxy

    restoreUnmaskedPrefs();
    return true;
    
}

window.addEventListener("load", initFoxtor, false);
